==> turbo-react-pdf-highlighter-3.3.2.tgz: package/src/lib/optimize-text-parts.js <==
import type { T_LTWH, T_VIEWPORT } from "../types.js";
import getBoundingRect from "./get-bounding-rect";
import { viewportToPdf } from "./coordinates";

const optimizeTextParts = (
	parts: Array<{ rect: T_LTWH, text: string }>,
	viewport: T_VIEWPORT
) => {
	console.log('optimizeTextParts data:', parts);
	console.log('optimizeTextParts viewport:', viewport);
	if (parts.length === 0) {
		console.log('optimizeTextParts result: empty');
		return {
			text: '',
			textParts: []
		};
	}
	
	const result = [];
	let last = parts[0];
	for (let i = 1; i < parts.length; i++) {
		const current = parts[i];

		if (current.rect.top < last.rect.top + last.rect.height) {
			const lastChar = last.text.length && last.text[last.text.length-1];
			const firstChar = current.text.length && current.text[0];
			const isAddSpace = lastChar && firstChar &&
				lastChar !== ' ' && lastChar !== '\t' && firstChar !== ' ' && firstChar !== '\t';

			last.rect = getBoundingRect([last.rect, current.rect]);
			last.text = last.text + (isAddSpace ? ' ' : '') + current.text
		} else {
			result.push(last);

			last = current;
		}
	}
	result.push(last);

	const [pageXOffset, pageYOffset, pageWidth, pageHeight] = viewport.viewBox;
	const res = {
		text: result.map(x => x.text).join('\n'),
		textParts: result.map(r => {
			const rect = r.rect;

			return { 
				text: r.text,
				rect: viewportToPdf(rect, viewport),
				usePdfCoordinates: true,
			};
		}),
		pageSize: { width: pageWidth, height: pageHeight },
	};
	console.log('optimizeTextParts result:', res);
	return res;
}


export default optimizeTextParts;

==> turbo-react-pdf-highlighter-3.3.11.tgz: package/src/lib/optimize-text-parts.js <==
import type { T_LTWH, T_VIEWPORT } from "../types.js";
import getBoundingRect from "./get-bounding-rect";
import { viewportToPdf } from "./coordinates";

const optimizeTextParts = (
	parts: Array<{ rect: T_LTWH, text: string }>,
	viewport: T_VIEWPORT
) => {
	console.log('optimizeTextParts data:', parts);
	console.log('optimizeTextParts viewport:', viewport);

	const sensitivity = 1 / viewport.scale;
	console.log('optimizeTextParts sensitivity:', sensitivity);

	if (parts.length === 0) {
		console.log('optimizeTextParts result: empty');
		return {
			text: '',
			textParts: []
		};
	}
	
	const result = [];
	let last = parts[0];
	for (let i = 1; i < parts.length; i++) {
		const current = parts[i];

		if (current.rect.top < last.rect.top + last.rect.height - sensitivity) {
			const lastChar = last.text.length && last.text[last.text.length-1];
			const firstChar = current.text.length && current.text[0];
			const isAddSpace = lastChar && firstChar &&
				lastChar !== ' ' && lastChar !== '\t' && firstChar !== ' ' && firstChar !== '\t';

			last.rect = getBoundingRect([last.rect, current.rect]);
			last.text = last.text + (isAddSpace ? ' ' : '') + current.text
		} else {
			result.push(last);

			last = current;
		}
	}
	result.push(last);

	const [pageXOffset, pageYOffset, pageWidth, pageHeight] = viewport.viewBox;
	const res = {
		text: result.map(x => x.text).join('\n'),
		textParts: result.map(r => {
			const rect = r.rect;

			return { 
				text: r.text,
				rect: viewportToPdf(rect, viewport),
				usePdfCoordinates: true,
			};
		}),
		pageSize: { width: pageWidth, height: pageHeight },
	};
	console.log('optimizeTextParts result:', res);
	return res;
}


export default optimizeTextParts;
